<?php
/*
Plugin Name: Ps User Profile Image
Plugin URI: http://www.web-strategy.jp/
Description: ユーザー管理画面からプロフィール画像がアップロードできるようになりますし。
Author: Hitoshi Omagari
Version: 0.1
Author URI: http://www.warna.info/
*/

class ps_user_profile_image {
	
	var $pagenow;
	var $default_image_size = array(
		'thumbnail' => array( 48, 48 ),
		'standard'  => array( 96, 96 ),
		'big'       => array( 120, 120 ),
	);

function ps_user_profile_image() {
	$this->__construct();
}

function __construct() {
	if ( is_admin() && preg_match( '/^(profile\.php|user-edit\.php)/', basename( $_SERVER['REQUEST_URI'] ) ) ) {
		$this->pagenow = parse_url( $_SERVER['REQUEST_URI'] );
		$this->pagenow = basename( $this->pagenow['path'] );

		add_action( 'user_edit_form_tag', array( &$this, 'add_enctype_attr' ) );
		add_action( 'show_password_fields', array( &$this, 'profile_image_form' ) );
		add_action( 'profile_update', array( &$this, 'upload_profile_image' ), 10, 2 );
		add_action( 'admin_head-' . $this->pagenow, array( &$this, 'check_profile_dir' ) );
	}
}


function check_profile_dir() {
	global $errors;

	$check_dirs = array(
		WP_CONTENT_DIR,
		WP_CONTENT_DIR . '/ps_profile_image',
		WP_CONTENT_DIR . '/ps_profile_image/tmp'
	);
	
	foreach ( $check_dirs as $dir ) {
		if ( ! file_exists( $dir ) && $dir != WP_CONTENT_DIR ) {
			mkdir( $dir );
		}
		if ( ! is_writable( $dir ) ) {
			if ( ! isset( $errors ) ) {
				$errors = new WP_Error();
			}
			$errors->add( 'ps_profile_image', '<strong>ERROR</strong>:' . esc_html( $dir ) . 'に書き込み権限がありません。' );
		}
	}
}


function check_user_profile_dir( $user_id ) {
	$user_dir = $this->get_user_profile_dir( $user_id );
	
	if ( ! file_exists( $user_dir ) ) {
		return mkdir( $user_dir );
	}
	if ( ! is_writable( $user_dir ) ) {
		return false;
	}
}


function profile_image_form ( $bool ) {
	global $profileuser;
	
	$uploads = wp_upload_dir();
	$size_key = array_shift( array_keys( $this->default_image_size ) );
	$img_src = $this->get_user_profile_image_src( $profileuser->ID, $size_key );
?>
    <tr>
        <th scope="row">プロフィール画像</th>
            <td>
				<?php if ( $img_src ) : $this->user_profile_image( $profileuser->ID, $size_key ); ?>
					<br />
				<?php endif; ?>
                <input type="file" name="ps_profile_image" id="ps_profile_image" size="50" /><br />
				<input type="checkbox" name="ps_profile_image_del" id="ps_profile_image_del" value="1" />
				<label for="ps_profile_image_del">プロフィール画像を消す</label>
            </td>
        </tr>
<?php
	return $bool;
}

function add_enctype_attr() {
    echo ' enctype="multipart/form-data"';
}


function upload_profile_image( $user_id, $old_user_data  ) {
	global $errors;
	
	if ( ! $this->check_user_profile_dir( $user_id ) ) {
		if ( ! isset( $errors ) ) {
			$errors = new WP_Error();
		}
		$errors->add( 'ps_profile_image', '<strong>ERROR</strong>:' . esc_html( $dir ) . 'が作成出来ないか、書き込み権限がありません。' );
	}
	
	if ( $_POST['ps_profile_image_del'] ) {
		$user = get_userdata( $user_id );

		if ( $user->ps_profile_image ) {
			delete_user_meta( $user_id, 'ps_profile_image' );
		}
		
		$user_profile_dir = $this->get_user_profile_dir( $user->ID );
		
		if ( $hdl = opendir( $user_profile_dir ) ) {
			while ( false !== ( $file = readdir( $hdl ) ) ) {
				if ( ! preg_match( '/^\./', $file ) ) {
					@unlink( $user_profile_dir . '/' . $file );
				}
			}
		}
		


	} elseif ( $file = $_FILES['ps_profile_image'] ) {
		if ( ! $file['error'] && file_is_displayable_image( $file['tmp_name'] ) && is_uploaded_file( $file['tmp_name'] ) ) {
			$user = get_userdata( $user_id );
			
			$basedir = WP_CONTENT_DIR . '/ps_profile_image';
			
			$imagesize = getimagesize( $file['tmp_name'] );
			switch ( $imagesize['mime'] ) {
				case 'image/jpeg' :
					$ext = 'jpg';
					break;
				case 'image/png' :
					$ext = 'png';
					break;
				case 'image/gif' :
					$ext = 'gif';
					break;
				default :
			}
			
			if ( file_exists( $dir ) ) {
			}
			$tmppath = 	$basedir . '/tmp/' . $user->user_login . '.' . $ext;
			@move_uploaded_file( $file['tmp_name'], $tmppath );

			$dest_path = $this->get_user_profile_dir( $user->ID );
			
			$profile_images = array();
			foreach ( $this->default_image_size as $size => $wh ) {
				$r = image_resize( $tmppath, $wh[0], $wh[1], true, $size, $dest_path );
				if ( $r ) {
					$profile_images[$size] = basename( $r );
				}
			}
			if ( count( $profile_images ) ) {
				update_user_meta( $user_id, 'ps_profile_image', $profile_images );
			}
			unlink( $tmppath );
			
		} else {

		}
	}
}


function get_user_profile_dir( $user_id ) {
	$user_id = (int)$user_id;
	return WP_CONTENT_DIR . '/ps_profile_image/' . $user_id;
}


function get_user_profile_ulr( $user_id ) {
	$user_id = (int)$user_id;
	return WP_CONTENT_URL . '/ps_profile_image/' . $user_id;
}


function get_user_profile_image_src( $user_id, $size ) {
	$user_id = (int)$user_id;
	$user = get_userdata( $user_id );
	if ( ! $user || ! $user->ps_profile_image[$size] ) { return false; }
	
	if ( ! file_exists( $this->get_user_profile_dir( $user_id ) . '/' . $user->ps_profile_image[$size] ) ) { return false; }
	
	return preg_replace( '|^https?://[^/]+/|', '/', $this->get_user_profile_ulr( $user_id ) . '/' . $user->ps_profile_image[$size] );
}


function user_profile_image( $user_id, $size ) {
	$user_id = (int)$user_id;
	$user = get_userdata( $user_id );
	$src = $this->get_user_profile_image_src( $user_id, $size );
	
	if ( $src ) {
		echo '<img src="' . $src . '" width="' . $this->default_image_size[$size][0] . '" height="' . $this->default_image_size[$size][1] . '" alt="' . esc_attr( $profileuser->display_name ) . '" />';
	}
}


} // class end


$ps_user_profile_image =& new ps_user_profile_image();


function ps_get_user_profile_image_src( $user_id, $size ) {
	global $ps_user_profile_image;
	return $ps_user_profile_image->get_user_profile_image_src( $user_id, $size );
}


function ps_user_profile_image( $user_id, $size ) {
	global $ps_user_profile_image;
	$ps_user_profile_image->user_profile_image( $user_id, $size );
}